#include <stdlib.h>
#include "gba.h"
#include "gbalib.h"

u16 bg0_width;
u16 bg0_height;

const u8 sin_table [360] = {
128, 130, 132, 134, 136, 139, 141, 143, 145, 148, 150, 152, 
154, 156, 158, 161, 163, 165, 167, 169, 171, 173, 175, 178, 
180, 182, 184, 186, 188, 190, 191, 193, 195, 197, 199, 201, 
203, 205, 206, 208, 210, 211, 213, 215, 216, 218, 220, 221, 
223, 224, 226, 227, 228, 230, 231, 232, 234, 235, 236, 237, 
238, 239, 241, 242, 243, 244, 244, 245, 246, 247, 248, 249, 
249, 250, 251, 251, 252, 252, 253, 253, 254, 254, 254, 255, 
255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 
255, 255, 254, 254, 254, 253, 253, 252, 252, 251, 251, 250, 
249, 249, 248, 247, 246, 245, 244, 244, 243, 242, 241, 239, 
238, 237, 236, 235, 234, 232, 231, 230, 228, 227, 226, 224, 
223, 221, 220, 218, 216, 215, 213, 211, 210, 208, 206, 205, 
203, 201, 199, 197, 195, 193, 192, 190, 188, 186, 184, 182, 
180, 178, 175, 173, 171, 169, 167, 165, 163, 161, 158, 156, 
154, 152, 150, 148, 145, 143, 141, 139, 136, 134, 132, 130, 
128, 125, 123, 121, 119, 116, 114, 112, 110, 107, 105, 103, 
101, 99, 97, 94, 92, 90, 88, 86, 84, 82, 80, 77, 
75, 73, 71, 69, 67, 65, 64, 62, 60, 58, 56, 54, 
52, 50, 49, 47, 45, 44, 42, 40, 39, 37, 35, 34, 
32, 31, 29, 28, 27, 25, 24, 23, 21, 20, 19, 18, 
17, 16, 14, 13, 12, 11, 11, 10, 9, 8, 7, 6, 
6, 5, 4, 4, 3, 3, 2, 2, 1, 1, 1, 0, 
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
0, 0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 5, 
6, 6, 7, 8, 9, 10, 11, 11, 12, 13, 14, 16, 
17, 18, 19, 20, 21, 23, 24, 25, 27, 28, 29, 31, 
32, 34, 35, 37, 39, 40, 42, 44, 45, 47, 49, 50, 
52, 54, 56, 58, 60, 62, 63, 65, 67, 69, 71, 73, 
75, 77, 80, 82, 84, 86, 88, 90, 92, 94, 97, 99, 
101, 103, 105, 107, 110, 112, 114, 116, 119, 121, 123, 125
};

void init_gbalib (void)
{
  bg0_width = 32;
  bg0_height = 32;

}



void mappy_debug (const char *sz) 
{
  asm volatile("
    mov r2, %0
    ldr r0, =0xc0ded00d
    mov r1, #0
    and r0, r0, r0
  " :
  /* No output */ :
  "r" (sz) :
  "r0", "r1", "r2");
}


void bg0_off (void)
{
  vsync();
  if (REG_DISPCNT & BG0_ENABLE)
    REG_DISPCNT = REG_DISPCNT - BG0_ENABLE;
}

void bg0_on (void)
{
  vsync();
  if (!(REG_DISPCNT & BG0_ENABLE))
    REG_DISPCNT = REG_DISPCNT + BG0_ENABLE;
}

void bg1_off (void)
{
  vsync();
  if (REG_DISPCNT & BG1_ENABLE)
    REG_DISPCNT = REG_DISPCNT - BG1_ENABLE;
}

void bg1_on (void)
{
  vsync();
  if (!(REG_DISPCNT & BG1_ENABLE))
    REG_DISPCNT = REG_DISPCNT + BG1_ENABLE;
}

void bg2_off (void)
{
  vsync();
  if (REG_DISPCNT & BG2_ENABLE)
    REG_DISPCNT = REG_DISPCNT - BG2_ENABLE;
}

void bg2_on (void)
{
  vsync();
  if (!(REG_DISPCNT & BG2_ENABLE))
    REG_DISPCNT = REG_DISPCNT + BG2_ENABLE;
}

void bg3_off (void)
{
  vsync();
  if (REG_DISPCNT & BG3_ENABLE)
    REG_DISPCNT = REG_DISPCNT - BG3_ENABLE;
}

void bg3_on (void)
{
  vsync();
  if (!(REG_DISPCNT & BG3_ENABLE))
    REG_DISPCNT = REG_DISPCNT + BG3_ENABLE;
}


// sets the background palette from u8 rgb pal data
void set_bkg_pal (u8 *data)
{
  u16 cnt = 0;
  u16 r,g,b;
  u16 c;

  for (c=0;c<256;c++)
  {
    r = data[cnt];
    r = (r >> 3);
    cnt++;
    g = data[cnt];
    g = g >> 3;
    g = g << 5;
    cnt++;
    b = data[cnt];
    b = b >> 3;
    b = b << 10;
    cnt++;
    
    r = r+g+b;
    REG_PALETTE_BASE[c] = r;
  }
}

void loadchardata(const u8 *chardata, u32 amount, u16 *startram)
{
u32 i=0;
u16 data=0;
u16 *tvram = startram;
do
 {
   data = *chardata;
   chardata++;
   data |= (*chardata<<8);
   *tvram = data;
   tvram++;
   chardata++;
   i+=2;
 }
while(i<amount);
}

void start_interrupts (void)
{


}

void end_interrupts (void)
{

}

// interrupts that do nothing can use this function
void null_fn (void)
{

}


u16 button_pressed (u16 j)
{
  if (!(j & J_A))
    return 1;
  if (!(j & J_B))
    return 1;
  if (!(j & J_R))
    return 1;
  if (!(j & J_L))
    return 1;
  if (!(j & J_START))
    return 1;
  if (!(j & J_SELECT))
    return 1;

  return 0;


}


void wait_for_button (void)
{
  u16 joy;
  
  joy = (volatile u16) REG_P1;

  while (button_pressed (joy))
  {
    vsync ();
    rand();
    joy = (volatile u16) REG_P1;
  }

  while (!button_pressed (joy))
  {
    vsync ();
    rand();
    joy = (volatile u16) REG_P1;
  }

  while (button_pressed (joy))
  {
    vsync ();
    rand();
    joy = (volatile u16) REG_P1;
  }

}

void wait_for_start (void)
{
  u16 joy;

  joy = (volatile u16) REG_P1;

  while (!(joy & J_START))
  {
    vsync ();
    rand();
    joy = (volatile u16) REG_P1;
  }

  while (joy & J_START)
  {
    vsync ();
    rand();
    joy = (volatile u16) REG_P1;
  }

  while (!(joy & J_START))
  {
    vsync ();
    rand();
    joy = (volatile u16) REG_P1;
  }

}

u8 glib_sin (u16 x)
{
  if ((x>0) && (x<360))
    return sin_table[x];
  else
  {
    x = x % 360;
    return sin_table[x];
  }
}

u8 glib_cos (u16 x)
{
  return glib_sin(x+90);
}

// sprite functions
void sprite_move (Sprite *s, u16 x, u16 y)
{
  u16 not_y = 0xFF00;
  u16 not_x = 0xFE00;
  
  x = x & 0x1FF;
  y = y & 0xFF;
  s->Attrib0 = s->Attrib0 & not_y;
  s->Attrib0 += y;
  
  s->Attrib1 = s->Attrib1 & not_x;
  s->Attrib1 += x;

}

// sets the sprite size
void sprite_size (Sprite *s, u8 new_size)
{
  u16 not_size = 0x3FFF;
  u16 size;
  if (new_size > 3) 
    return;

  size = new_size ;
  size = size << 14;
  s->Attrib1 = s->Attrib1 & not_size;
  s->Attrib1 += size;
  
}

// sets the sprite hflip flag
void sprite_hflip (Sprite *s, u8 on_off)
{
  u16 not_hflip = 0xefFF;
  u16 hf;
  if (on_off > 1) 
    return;

  hf = on_off ;
  hf = hf << 12;
  s->Attrib1 = s->Attrib1 & not_hflip;
  s->Attrib1 += hf;
  
}

// sets the sprite shape
void sprite_shape (Sprite *s, u8 new_shape)
{
  u16 not_shape = 0x3FFF;
  u16 shape;
  if (new_shape > 3) 
    return;

  shape = new_shape ;
  shape = shape << 14;
  s->Attrib0 = s->Attrib0 & not_shape;
  s->Attrib0 += shape;
  
}

void sprite_tile (Sprite *s, u16 tile)
{
  u16 not_tile = 0xFC00;

  if (tile > 1024)
    return;

  s->Attrib2 = s->Attrib2 & not_tile;
  s->Attrib2 += tile;

}


/*
  SRAM Functions
*/


u8 *sram = (u8*)0x0e000000;
u16 sram_at = 0;

void init_sram (u16 pos)
{
  sram_at = pos;
}


u8 read_sram_8 (void)
{
  u8 t;

  t = sram[sram_at];
  sram_at++;

  return t;
}

void write_sram_8 (u8 t)
{
  sram[sram_at] = t;
  sram_at++;
}

u16 read_sram_16 (void)
{
  u16 t;

  t = sram[sram_at];
  t = t <<8;
  sram_at++;

  t += sram[sram_at];
  sram_at++;

  return t;
}

void write_sram_16 (u16 t)
{
  u8 x;

  x = t >> 8;
  sram[sram_at] = x;
  sram_at++;

  x = t & 0xff;
  sram[sram_at] = x;
  sram_at++;
}

u32 read_sram_32 (void)
{
  u32 t;

  t = sram[sram_at];
  t = t <<8;
  sram_at++;
 
  t += sram[sram_at];
  t = t <<8;
  sram_at++;

  t += sram[sram_at];
  t = t <<8;
  sram_at++;

  t += sram[sram_at];
  sram_at++;

  return t;
}

void write_sram_32 (u32 t)
{
  u8 x;
  u32 t2;
  t2 = t >> 24;
  x = t2;
  sram[sram_at] = x;
  sram_at++;

  t2 = t & 0x00ffffff;
  t2 = t2 >> 16;
  x = t2;
  sram[sram_at] = x;
  sram_at++;

  t2 = t & 0x0000ffff;
  t2 = t2 >> 8;
  x = t2;
  sram[sram_at] = x;
  sram_at++;

  t2 = t & 0x000000ff;
  x = t2;
  sram[sram_at] = x;
  sram_at++;
}


