#ifndef GBALIB_HEADER
#define GBALIB_HEADER

void start_interrupts (void);

// wait until vsync starts
#define vsync() while (REG_VCOUNT != 160);

// prints a debug string in mappy
void mappy_debug (const char *sz);

// initialize all the gbalib variables
void init_gbalib (void);

// Turn bg0 off
void bg0_off (void);

// turn bg0 on
void bg0_on (void);

// Turn bg1 off
void bg1_off (void);

// turn bg1 on
void bg1_on (void);

// Turn bg2 off
void bg2_off (void);

// turn bg2 on
void bg2_on (void);

// Turn bg3 off
void bg3_off (void);

// turn bg3 on
void bg3_on (void);

// turn on and off interrupts
void start_interrupts (void);
void end_interrupts (void);

// interrupts that do nothing can use this function
void null_fn (void);


// 1 a button is being pressed
// 0 no button is being pressed
u16 button_pressed (u16 j);

// wait for a button to be pressed
void wait_for_button (void);

// wait for the start button to be pressed
void wait_for_start (void);

// sets the background palette from u8 rgb pal data
void set_bkg_pal (u8 *data);

// load some tile or sprite data to the given location
void loadchardata(const u8 *chardata, u32 amount, u16 *startram);

// sin of angle x (degrees)
u8 glib_sin (u16 x);

// cos of angle x (degrees)
u8 glib_cos (u16 x);

/////////////////////////////////////////////////////////////////////
// sprite functions
/////////////////////////////////////////////////////////////////////

// sets the x and y co ordinates of sprite num
void sprite_move (Sprite *s, u16 x, u16 y);

// sets the sprite size
void sprite_size (Sprite *s, u8 size);

// sets the sprite shape
void sprite_shape (Sprite *s, u8 shape);

// sets the sprite image to start at a given tile
void sprite_tile (Sprite *s, u16 tile);

// sets the sprite hflip flag
void sprite_hflip (Sprite *s, u8 on_off);

//////////////////////////////////////////////////////////////////////
// SRAM functions
//////////////////////////////////////////////////////////////////////

void init_sram (u16 pos);

u8 read_sram_8 (void);
u16 read_sram_16 (void);
u32 read_sram_32 (void);

void write_sram_8 (u8 t);
void write_sram_16 (u16 t);
void write_sram_32 (u32 t);
#endif
